;; Programa: Package con agentes jugadores simples de PlanTron
;;
;; Asignatura: Introducción a la Inteligencia Artificial
;;             ETSII - ULL
;; Práctica: (lisp)
;; Profesor: Patricio García (patricio AT etsii.ull.es)
;; Fecha:    28/04/2008
;; Comentarios:
;;
;;           Incluye agentes para jugar al PlanTron simples, esto
;;           es, agentes que no tiene memoria y su comportamiento
;;           viene guiado en función del entorno que perciben en
;;           cada instante (excepto el agente-hormiga-borracha,
;;           que hace uso del generador de números aleatorios).
;;
;;           Los agentes definidos utilizan el entorno de juego
;;           PlanTron, definido principalmente por los packages
;;           PT-BASE y ENVIO.
;;
;;           Los agentes incluidos son, ordenados según la
;;           complejidad de su código, los siguientes:
;;
;;             AGENTE-LADOS
;;             AGENTE-ESQUINA
;;             AGENTE-ESPIRAL
;;             AGENTE-BORDES
;;             AGENTE-SIN-MIEDO
;;             AGENTE-COBARDE
;;             AGENTE-HORMIGA-BORRACHA
;;
;;           Ver la documentación existente para cada uno de ellos.
;;
;;           Para su utilización es aconsejable usar el cargador
;;           del entorno de juego PlanTron: "pt-loader.lisp".
;;
;;
;; Dependencias:
;;
;;           Hace uso del paquete que contiene la interface (ENVIO)
;;           con el control del juego (PT-BASE).
;;
;;
;; Modificaciones:
;;
;;           - Ver 0.1 (28/04/2008): Primera versión
;;
;;           - Ver 0.2 (14/04/2009): Versión curso 2008/09
;;
;;           - Ver 0.2.1 (15/04/2009): Añadido más comentarios
;;
;; Por hacer:
;;
;;
;;


;;
;; Definición del package
;; ----------------------

(defpackage "PT-AGENTES-SIMPLES"
  (:nicknames "PT-A-S")
  (:use "COMMON-LISP")
  (:export "VERSION" "DIR-AGENTE-P" "DIR-POSIBLE-P" "DIR-INC" "DIR-POSIBLE-PREF" "LIS-DIR-POSIBLE-P")
  (:documentation "Agentes para jugar al PlanTron simples, esto es, agentes que no tiene memoria y su comportamiento viene guiado en función del entorno que perciben en cada instante (excepto el agente-hormiga-borracha, que hace uso del generador de números aleatorios)."))

(in-package "PT-AGENTES-SIMPLES")


;;
;; Constante versión
;; -----------------

(defconstant version "0.2.1"
  "Versión del package PT-AGENTES-SIMPLES.")


;;
;; Funciones de tratamiento de direcciones y posiciones
;; ----------------------------------------------------

(defun dir-agente-p (pos dir)
  "Mira si hay un agente en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (eq (envio:consulta-tipo                ; Consulta tipo de contenido
	   (envio:consulta-pos-tab pos dir))  ; Consulta contenido tablero POS+DIR
	  'envio:jugador)                     ; Tipo agente jugador
      dir))                                   ; Si hay agente devuelve DIR

(defun dir-posible-p (pos dir)
  "Mira si es posible colocarse (planta o vacío) en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (member (envio:consulta-tipo (envio:consulta-pos-tab pos dir))
	      '(envio:planta nil))            ; Revisa si el contenido es planta
      dir))                                   ; o vacío entonces devolviendo DIR

(defun number-o-0 (elm)
  "Si ELM es un número lo devuelve, sino devuelve 0."
  (if (numberp elm)
      elm
    0))

(defun dir-inc (dir dir-ini)
  "Calcula la dirección DIR relativa a DIR-INI, esto es, gira DIR-INI pasos en sentido agujas del reloj sobre DIR. DIR y DIR-INI han de ser números enteros (direcciones, número entero entre 0 y 3: norte, este, sur y oeste respectivamente), si no se consideran cero (norte)."
  (rem (+ (number-o-0 dir)
	  (number-o-0 dir-ini))
       4))

(defun dir-posible-pref (pos lis-dir-pref &optional dir-ini)
  "Busca la primera dirección posible (ver función DIR-POSIBLE-P) alrededor de POS siguiendo el orden de LIS-DIR-PREF, si DIR-INI está instanciado las direcciones de LIS-DIR-PREF son relativas a DIR-INI. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. LIS-DIR-PREF ha de ser una lista con números enteros (direcciones). DIR-INI ha de ser NIL, direccionamiento absoluto, o un número entero (dirección), direccionamiento relativo al valor de DIR-INI."
  (if (consp lis-dir-pref)
      (or (dir-posible-p pos (dir-inc (first lis-dir-pref) dir-ini))
	  (dir-posible-pref pos (rest lis-dir-pref) dir-ini))))


;;
;; Agente OSEN
;; -----------

(defun agente-lados (id)
  "Agente jugador PlanTron que se va por los lados."
  (dir-posible-pref (envio:consulta-pos-jug id) '(3 2 1 0)))


;;
;; Agente lados opuestos
;; ---------------------

(defun suma-a-todos (num lis-num)
  "Genera lista a partir de sumar NUM a todos los elementos de LIS-NUM"
  (cond
   ((endp lis-num) nil)
   (t (cons (+ num (first lis-num))
	    (suma-a-todos num (rest lis-num))))))

(defun agente-esquina (id)
  "Agente jugador PlanTron que se va cada uno por su lado."
  (dir-posible-pref (envio:consulta-pos-jug id)    ; Permite iniciar en distintas
		    (suma-a-todos id '(3 2 1 0)))) ; direcciones según ID


;;
;; Agente espiral derecha
;; ----------------------
  
(defun agente-espiral (id)
  "Agente jugador PlanTron que da vueltas en espiral desenrrollándose!"
  (dir-posible-pref (envio:consulta-pos-jug id)  ; Der, Frente, Izq, Atrás
		    '(1 0 3 2)                   ; Lista de direcciones relativa a
		    (first (envio:consulta-est-jug id)))) ; la dirección última


;;
;; Agente bordes
;; -------------

(defun agente-bordes (id)
  "Agente jugador PlanTron que va temerariamente por los bordes"
  (dir-posible-pref (envio:consulta-pos-jug id)
		    '(0 1 3 2)                    ; Frente, Der, Izq, Atrás
		    (first (envio:consulta-est-jug id))))


;;
;; Agente sin-miedo
;; ----------------

(defun alejarse-p (pos lis-dir-pref)
  "Revisa posiciones de LIS-DIR-PREF y si está pegado a algún agente trata de escapar. Si no tiene agente pegado no se mueve"
  (cond
   ((endp lis-dir-pref) nil)
   ((and (dir-agente-p pos (first lis-dir-pref))
	 (dir-posible-pref pos  '(2 1 3) (first lis-dir-pref))))
   (t (alejarse-p pos (rest lis-dir-pref)))))

(defun agente-sin-miedo (id)
  "Agente jugador PlanTron que solo cuando se le acerca otro agente trata de mover alejándose de él con un paso en otra dirección."
  (alejarse-p (envio:consulta-pos-jug id) '(0 2 1 3)))


;;
;; Agente cobarde
;; --------------

(defun huir-p (id pos lis-dir-pref)
  "Revisa posiciones de LIS-DIR-PREF y si está pegado a algún agente trata de escapar. Si no tiene agente pegado trata de moverse com la útima vez."
  (cond
   ((endp lis-dir-pref)
    (dir-posible-p pos (first (envio:consulta-est-jug id))))
   ((and (dir-agente-p pos (first lis-dir-pref))
	 (dir-posible-pref pos  '(2 1 3) (first lis-dir-pref))))
   (t (huir-p id pos (rest lis-dir-pref)))))

(defun agente-cobarde (id)
  "Agente jugador PlanTron que en cuanto se le acerca otro agente trata de salir corriendo en otra dirección."
  (huir-p id (envio:consulta-pos-jug id) '(0 2 1 3)))


;;
;; Agente hormiga borracha
;; -----------------------

(defvar random-state-hormiga-borracha (make-random-state *random-state*)
  "Estado random para el AGENTE-HORMIGA-BORRACHA.")

(defun lis-dir-posible-p (pos lis-dir-pref)
  "Genera una lista con las direcciones a las que es posible ir a partir de las direcciones de LIS-DIR-PREF"
  (cond
   ((endp lis-dir-pref) nil)
   ((dir-posible-p pos (first lis-dir-pref))
    (cons (first lis-dir-pref) (lis-dir-posible-p pos (rest lis-dir-pref))))
   (t (lis-dir-posible-p pos (rest lis-dir-pref)))))

(defun elemento-aleatorio (lista)
  "Elige un elemento aleatorio de la lista LISTA"
  (nth (random (length lista) random-state-hormiga-borracha) lista))

(defun agente-hormiga-borracha (id)
  "Agente jugador PlanTron que da tumbos y se agarra a las farolas. Elige unmovimientro aleatoriamente entre los posibles (quedarse quieto y direcciones vacías a su alrededor)."
  (elemento-aleatorio
   (cons nil            ; Incluir posiblidad de no moverse
	 (lis-dir-posible-p (envio:consulta-pos-jug id) '(0 1 2 3)))))


;;
;; Definiciones de agentes presentes en este package
;; -------------------------------------------------

(pt:defagente lados
  :turno #'agente-lados
  :documentation "Agente jugador PlanTron que se va por los lados")

(pt:defagente esquina
  :turno #'agente-esquina
  :documentation "Agente jugador PlanTron que se va cada uno por su lado.")

(pt:defagente espiral
  :turno #'agente-espiral
  :documentation "Agente jugador PlanTron que da vueltas en espiral desenrrollándose!")

(pt:defagente bordes
  :turno #'agente-bordes
  :documentation "Agente jugador PlanTron que va temerariamente enrrollándose!")

(pt:defagente sin-miedo
  :turno #'agente-sin-miedo
  :documentation "Agente jugador PlanTron que solo cuando se le acerca otro agente se trata de mover alejándose de él con un paso en otra dirección")

(pt:defagente cobarde
  :turno #'agente-cobarde
  :documentation "Agente jugador PlanTron que en cuanto se le acerca otro agente trata de salir corriendo en otra dirección")

(pt:defagente hormiga-borracha
  :turno #'agente-hormiga-borracha
  :documentation "Agente jugador PlanTron que da tumbos y se agarra a las farolas. Elige unmovimientro aleatoriamente entre los posibles (quedarse quieto y direcciones posibles a su alrededor)")
