;; --------------------------------
;; Presentación de entorno PlanTron
;; --------------------------------
;;  Este fichero recoge una serie de instrucciones
;;  que guían el aprendizaje del entorno de agentes
;;  PlanTron (ver fichero README).
;;
;;  Puedes ir copiando sobre una consola las
;;  instrucciones aquí descritas.
;;


;; Para cargar el entorno
;; ----------------------

;; Arrancamos en consola el CMUCL lisp con 'cmucl'
#|

cmucl

|#
;; También es posible usar SBCL lisp

;; Para salir del lisp usar la función 'quit':
#|

(quit)

|#


;; Cargamos el entorno en el lugar en que esté,
;; en el CC de la ETSII se encuentra en el
;; directorio /scratch/plantron
;;

(load "/scratch/plantron/pt-loader")

;; ¡Atención! Cargar siempre desde scratch, ya
;; que pueden haber modificaciones en los códigos.
;; Comprobar versión contenido fichero VERSION o
;; ejecutar (version-pt)



;; Partida con humano
;; ------------------

;; Movimientos de teclas (teclado numérico):
;;   8: arriba, 2:abajo, 6: derecha, 4:izquierda

;; ¡Atención! Asegurarse antes de jugar que el
;; tamaño de la terminal es al menos de 80x24
;; (VT100)
;; Trata de que no te maten

(pt:partida)


;; La expresión anterior es equivalente a:

(pt:partida "HUMANO")


;; Descripción de funciones y variables con describe

(describe 'pt:partida)


;; Información del agente

(pt:info-jug "HUMANO")



;; Partida con agente lados
;; ------------------------

(pt:partida "LADOS")


;; Más rápido, opción :sleep-cs <centesimas de segundo/turno>

(pt:partida "LADOS" :sleep-cs 1)


;; Utilización permanente de :sleep-cs

(setf pt-base:*sleep-cs* 15)



;; Cambio del agente depredador
;; ----------------------------

;; Agente lados contra agente depredador humano

(pt:partida "LADOS" :gen-dir "HUMANO-DPDD")


;; Agente lados contra agente depredador humano
;; sin movimiento continuo

(pt:partida "LADOS" :gen-dir "HUMANO-DPDD2")


;; Agente hormiga borraca contra agente depredador humano
;; sin movimiento continuo

(pt:partida "HORMIGA-BORRACHA" :gen-dir "HUMANO-DPDD2")


;; Sin agente depredador

(pt:partida "HORMIGA-BORRACHA" :gen-dir NIL :sleep-cs 5)



;; Consultar jugadores disponibles
;; -------------------------------

(pt:jug)

(describe 'pt:jug)

(pt:jug :full t)



;; Partida con agente esquina
;; --------------------------

(pt:partida "ESQUINA" :sleep-cs 1)



;; Jugar con réplicas de agentes
;; -----------------------------

;; Jugar con 3 réplicas del agente esquina

(pt:partida "ESQUINA" :sleep-cs 5 :rep 3)


;; Fijar permanentemente 3 réplicas

(setf pt:*rep* 3)

(pt:partida "ESQUINA" :sleep-cs 5)



;; Partida con agente bordes y espiral
;; -----------------------------------

(pt:partida "BORDES" :sleep-cs 5)

(pt:partida "ESPIRAL" :sleep-cs 5)



;; Partida con agentes sin-miedo y cobarde
;; ---------------------------------------

(pt:partida "SIN-MIEDO" :sleep-cs 5 :rep 5)

(pt:partida "COBARDE" :sleep-cs 5 :rep 5)


;; Contra agente depredador humano
;; sin movimiento continuo

(pt:partida "COBARDE" :rep 5 :gen-dir "HUMANO-DPDD2")



;; Partidas con mas energía inicial
;; --------------------------------

;; Fijar permanentemente la energía inicial a
;; la mitad de las casillas del tablero

(setf pt-base:*ratio-ene-jug-ini* 0.5)

(pt:partida "SIN-MIEDO" :sleep-cs 5 :rep 5)

(pt:partida "COBARDE" :sleep-cs 5 :rep 5)



;; Cargar y ejecutar un nuevo agente
;; ---------------------------------

(pt:load-ag "agentes-aluXXXX.lisp")

(describe 'pt:load-ag)

(pt:jug)

(pt:partida "ALUXXXX-LADOS" :sleep-cs 1)



;; Partidas cambiando posiciones de inicio
;; ---------------------------------------

;; Posiciones de inicio aleatorias

(pt:partida "SIN-MIEDO" :rep 5 :pos-alea t :sleep-cs 30)


;; Posiciones de inicio no aleatorias

(pt:partida "SIN-MIEDO" :rep 5 :pos-alea nil)



;; Partidas con dos o mas agentes
;; ------------------------------

(pt:partida "ESQUINA" "ESPIRAL" "HORMIGA-BORRACHA" :sleep-cs 1)

(pt:partida "BORDES" "SIN-MIEDO" "COBARDE" :sleep-cs 1)



;; Partida con trazas de consultas, opción :vis <nivel>
;; ----------------------------------------------------

(pt:partida "COBARDE" :sleep-cs 10 :vis 2)

(pt:partida "COBARDE" :sleep-cs 1 :vis 2)


;; Sin visualizador

(pt:partida "COBARDE" :sleep-cs 1 :vis nil)


;; Utilización permanente de un visualizador

(setf pt:*vis* 2)

(pt:partida "COBARDE" :sleep-cs 1)



;; Pintar el tablero en otra terminal, opción :dev-vis
;; ---------------------------------------------------

;; Abrir una nueva terminal de al menos de 80x24 (VT100),
;; localizar con comando 'w' o 'ps' el dispositivo
;; asociado a nueva terminal (/dev/pts/?),
;; por ejemplo para "/dev/pts/2"

(pt:partida "ESPIRAL" :sleep-cs 1 :dev-vis "/dev/pts/2")


;; Traceamos función dir-posible-pref

(trace pt-agentes-simples::dir-posible-pref)

(pt:partida "ESPIRAL" :sleep-cs 1 :dev-vis "/dev/pts/2")

(untrace  pt-agentes-simples::dir-posible-pref)


;; Utilización permanente de otra terminal

(setf pt:*dev-vis* "/dev/pts/2")

(pt:partida "ESPIRAL" :sleep-cs 1)



;; Partidas paso a paso
;; --------------------

;; Activación de paso a paso

(setf pt-vis:*step* t)

(pt:partida "ESPIRAL" :vis 2 :dev-vis "/dev/pts/2")

(pt:partida "COBARDE" :vis 2 :dev-vis "/dev/pts/2")

(describe 'pt-vis:*step*)


;; Desactivación de paso a paso

(setf pt-vis:*step* nil)



;; Partidas de todos los agentes no interactivos
;; ---------------------------------------------

(pt:partida-todos)

(describe 'pt:partida-todos)


;; No esperar entre turnos (acelera las partidas)

(pt:partida-todos :sleep-nil t)


;; Resultados de 5 partidas del agente espiral
;; ejecutadas sin visualizador

(pt:partida-todos '("HORMIGA-BORRACHA") :num 5 :vis nil :sleep-nil t)

;; El resultado consta de sublistas para cada agente que
;; indican nombre, tiempo acumulado de vida, energía
;; acumulada en los finales de partida y número de
;; partidas jugadas



;; Partida con otros generadores de plantas, opción :gen-pla
;; --------------------------------------------------------

;; Generador de plantas aleatorio

(pt:partida "COBARDE" :sleep-cs 1 :gen-pla 2 :vis 2)


;; Generador de plantas en cúmulos incremental,
;; detener con Ctrl-C y comando A

(pt:partida "BORDES" :sleep-cs 1 :gen-pla 3 :vis 2)



;; Envío de agentes: trabajo con repositorio arena
;; -----------------------------------------------

;; Mostrar los agentes en repositorio arena

(pt-arena:ls)

(describe 'pt-arena:ls)

(pt-arena:ls "*" t)


;; Chequear y enviar agentes al repositorio arena
;; ¡Atención! Asegurarse de ejecutar esta instrucción
;; desde la máquina tonga (exthost) y cargar siempre 
;; el entorno desde scratch (/scratch/plantron/pt-loader)

(pt-arena:send "agentes-aluXXXX.lisp")

;; Chequear cuidadosamente los mensajes y warnings emitidos
;; y asegurarse de que el fichero de agentes aparece listado
;; en el entorno arena. Si es necesario se puede reenviar
;; nuevamente el fichero borrándolo previamente mediante
;; pt-arena:del

(pt-arena:ls)


;; Cargar agente del repositorio arena

(pt-arena:load-ag "agentes-aluXXXX")


;; Eliminar un agente del repositorio arena
;; (no anula el envío)

(pt-arena:del "agentes-aluXXXX")

(describe 'pt-arena:del)


;; Tabla de resultados

(pt-arena:hof)

(describe 'pt-arena:hof)



;; Ejemplo de agentes buscadores de caminos
;; ----------------------------------------

; Cargar agentes de caminos del repositorio arena

(pt-arena:load-ag "agentes-aluXXXX-camino")


; Lanzar agente búsqueda de camino en anchura

(pt:partida "ALUXXXX-CAMINO-BA")

(pt:partida "ALUXXXX-CAMINO-BA" :vis 2)


; Lanzar agente búsqueda de camino en A*

(pt:partida "ALUXXXX-CAMINO-A" :vis 2)


; Lanzar 10 réplicas de agente búsqueda de camino en A*

(pt:partida "ALUXXXX-CAMINO-A" :rep 10 :sleep-nil t)



;; Compilación de agentes
;; ----------------------

(compile-file "agentes-aluXXXX-camino.lisp")

(pt:load-ag "agentes-aluXXXX-camino.x86f")



;; Salir del lisp
;; --------------

(quit)

